==> GeneticsBase_1.0.0.tar.gz: GeneticsBase/R/plot.LD.R <==
# $Id: plot.LD.R 1110 2006-11-21 22:13:37Z warnes $

plot.LD <- function(x, y=NULL, cex, ...)
  {
    # Color for text labels for LDtable
    textcol<-"black"
    # Number of significant digits to display
    digits<-3
    # Name(s) of LD information items to be displayed in the
    # upper triangle. Deafults to $R^2$
    upper<-c("R^2")
    # Name(s) of LD information items to be displayed in the
    #lower triangle. Defaults to P-value
    lower<-c("P-value")
    # cutoffs points for the upper triangle
    colorcut.upper<-c(0.00,  .25 , 0.50, 0.80 , 0.90 , 0.95, 1.00)
    # cutoffs points for the lower triangle
    colorcut.lower<-c(0.00, 0.001, 0.01, 0.025, 0.05 , 0.1, 0.5, 1.00)
    # Color scheme for the upper triangle 
    colors.upper<-colorpanel(length(colorcut.upper)-1, "snow", "blue")
    # Color scheme for the loer triangle
    colors.lower<-colorpanel(length(colorcut.lower)-1, "red", "snow")
    # Name of LD information items to be displayed in the
    #    upper triangle. Defaults to $R^2$ 
    colorize.upper<-"R^2"
    # Name of LD information items to be displayed in the
    # lower triangle. Defaults to P-value
    colorize.lower<-"P-value"

    if(! colorize.upper %in% slotNames(x))
      stop(colorize.upper, " not an element of ", deparse(substitute(x)) )

    if(! colorize.lower %in% slotNames(x))
      stop(colorize.lower, " not an element of ", deparse(substitute(x)) )

    which=c(upper, lower)
    datatab <- summary(x)
    
    missmatch <- which[!(which %in% slotNames(x))]
    if(length(missmatch)>0)
      stop(missmatch, " not an element of ", deparse(substitute(x)) )

    oldpar <- par( c("cex","mar","mfrow") )
    on.exit( par(oldpar) )

    par(mar=c(b=0.1,l=4.1,t=5.1,r=1.1))
    
    ## Setup plot region
    lmat <- cbind( c(1,2), c(1,3))
    layout(lmat, height=c(10,1.5))
    
    matform <- function( value, template )
      {
        dim(value) <- dim(template)
        dimnames(value) <- dimnames(template)
        value
      }
    
    tmp.upper <- cut(slot(x,colorize.upper), colorcut.upper, include.lowest=TRUE)
    colormat.upper <- matform(as.numeric(tmp.upper), slot(x,colorize.upper) )
    n.upper <- matform( paste("(",x@n,")",sep="" ), x@n)

    tmp.lower <- cut(slot(x,colorize.lower), colorcut.lower, include.lowest=TRUE)
    colormat.lower <- t(matform(as.numeric(tmp.lower), slot(x,colorize.lower) ))
    n.lower <- t(matform( paste("(",x@n,")",sep="" ), x@n))

    nmarker <- ncol(colormat.upper)

    colormat.final <- matrix(NA, nrow=nmarker, ncol=nmarker)
    dimnames(colormat.final) <- dimnames(colormat.upper)

    lower.tri <- (row(colormat.upper) - col(colormat.upper)) > 0
    upper.tri <- (col(colormat.upper) - row(colormat.upper)) > 0

    colormat.final[upper.tri] <- as.numeric(colormat.upper[upper.tri])
    colormat.final[lower.tri] <- as.numeric(colormat.lower[lower.tri])+ length(colors.upper)

    #
    # color coded frame boxes
    #

    colors.final <- c(colors.upper, colors.lower)
    image(x=1:nmarker, y=1:nmarker,
          z=t(colormat.final[nmarker:1,]),
          breaks=0:length(colors.final)+0.5,
          col=colors.final, xlab="", ylab="",
          xaxt="n", yaxt="n",...)

    abline(v=-0.5 + 1:(nmarker+1))
    abline(h=-0.5 + 1:(nmarker+1))
    
    
    #
    # text in boxes
    #

    if(missing(cex))
      {
        maxheight <- max(length(lower),
                         length(upper)) *
                     strheight('W')*1.05
        maxlength <- max(sapply(lower,nchar),
                         sapply(upper,nchar)) *
                     strwidth('W')*1.05
                
        cex <- min( 1 / maxheight, 1 / maxlength ) * 1.25
      }

    par(cex=cex)

    cin <- par("cin")

    axis(3, 1:nmarker, colnames(colormat.final))
    axis(2, 1:nmarker, rev(rownames(colormat.final)))

    lineheight <- strheight("W")*1.05
    
    showtext <- function(which, upper=TRUE)
      {
        center <- lineheight * length(which)/2

        for(i in 1:length(which))
          {
            displaymat <- slot(x,which[i])
            
            ## Put the items in lower.triangle, specified by user
            if(!upper) displaymat <- t(displaymat)
            
            if( which[i]=="P-value" )
              displaymat <- format.pval(displaymat, digits=digits)
            else if (which[i]!="n")
              displaymat <- format(round(displaymat, digits=digits),digits=digits)
            
            displaymat[] <- gsub("NA.*", "", as.character(displaymat))
            
            text(x=col(colormat.final),
                 y=nmarker - row(colormat.final)+ 1 + center - lineheight*(i-1),
                 displaymat,
                 col=textcol,
                 adj=c(0.5, 1)
                 )
          }
      }
    showtext(which=upper, upper=TRUE)
    showtext(which=lower, upper=FALSE)
    

    diag.names <- diag(1, nrow=nmarker, ncol=nmarker)
    
    text(x=1:nmarker, y=nmarker:1,
         rownames(colormat.final), adj=c(0.5,0.5),
         col=textcol)

    par(cex=oldpar$cex)
    
    #
    # title
    #
    title(main="Linkage Disequilibrium\n")


    ## Color scale calculations
    levs.lower <- levels(tmp.lower)
    levs.lower <- gsub( ",", ",\n ", levs.lower)
    
    levs.upper <- levels(tmp.upper)
    levs.upper <- gsub( ",", ",\n ", levs.upper)

    ##
    ## Lower color scale
    ##
    par(cex=1.0)
    par(mar=c(b=1.1, l=4.1, t=1.1, r=0.5))

    vals.lower <- 1:nlevels(tmp.lower)
    image(z=matrix(vals.lower,ncol=1), col=colors.lower, xaxt="n", yaxt="n")

    width <- par("usr")[2] - par("usr")[1] 
    cex.scale <- min(
                     c( 0.95 / length(levs.lower) / strwidth(levs.lower) ,
                        0.95 / length(levs.upper) / strwidth(levs.upper) )
                     )

    top    <- par("usr")[4]
    bottom <- par("usr")[3]
    center <- (top - bottom)/2 + bottom 
    
    text(x=(vals.lower-1)/(length(vals.lower)-1),
         y=rep(center,length(vals.lower)),
         labels=levs.lower, adj=c(0.5,0.5), cex=cex.scale,
         col=textcol)
    mtext(lower, side=3, las=1.1, cex=0.75)

    ##
    ## Upper color scale 
    ## 
    par(cex=1.0)
    par(mar=c(b=1.1, l=1.1, t=1.1, r=1.1))

    vals.upper <- 1:nlevels(tmp.upper)
    image(z=matrix(vals.upper,ncol=1), col=colors.upper, xaxt="n", yaxt="n")

    text(x=(vals.upper-1)/(length(vals.upper)-1),
         y=rep(center,length(vals.upper)),
         labels=levs.upper, adj=c(0.5,0.5), cex=cex.scale,
         col=textcol)
    mtext(upper, side=3, las=1, cex=0.75)
    
    invisible(colormat.final)
  }

if(!isGeneric("plot"))
  setGeneric("plot", function(x, y,...)
           standardGeneric("plot"))

setMethod("plot", "LD", plot.LD)

==> GeneticsBase_1.14.0.tar.gz: GeneticsBase/R/plot.LD.R <==
# $Id: plot.LD.R 1223 2007-05-26 16:20:28Z warnes $

plot.LD <- function(x, y=NULL, cex, ...)
  {
    # Color for text labels for LDtable
    textcol<-"black"
    # Number of significant digits to display
    digits<-3
    # Name(s) of LD information items to be displayed in the
    # upper triangle. Deafults to $R^2$
    upper<-c("R^2")
    # Name(s) of LD information items to be displayed in the
    #lower triangle. Defaults to P-value
    lower<-c("P-value")
    # cutoffs points for the upper triangle
    colorcut.upper<-c(0.00,  .25 , 0.50, 0.80 , 0.90 , 0.95, 1.00)
    # cutoffs points for the lower triangle
    colorcut.lower<-c(0.00, 0.001, 0.01, 0.025, 0.05 , 0.1, 0.5, 1.00)
    # Color scheme for the upper triangle 
    colors.upper<-colorpanel(length(colorcut.upper)-1, "snow", "blue")
    # Color scheme for the loer triangle
    colors.lower<-colorpanel(length(colorcut.lower)-1, "red", "snow")
    # Name of LD information items to be displayed in the
    #    upper triangle. Defaults to $R^2$ 
    colorize.upper<-"R^2"
    # Name of LD information items to be displayed in the
    # lower triangle. Defaults to P-value
    colorize.lower<-"P-value"

    if(! colorize.upper %in% slotNames(x))
      stop(colorize.upper, " not an element of ", deparse(substitute(x)) )

    if(! colorize.lower %in% slotNames(x))
      stop(colorize.lower, " not an element of ", deparse(substitute(x)) )

    which=c(upper, lower)
    datatab <- summary(x)
    
    missmatch <- which[!(which %in% slotNames(x))]
    if(length(missmatch)>0)
      stop(missmatch, " not an element of ", deparse(substitute(x)) )

    oldpar <- par( c("cex","mar","mfrow") )
    on.exit( par(oldpar) )

    par(mar=c(b=0.1,l=4.1,t=5.1,r=1.1))
    
    ## Setup plot region
    lmat <- cbind( c(1,2), c(1,3))
    layout(lmat, height=c(10,1.5))
    
    matform <- function( value, template )
      {
        dim(value) <- dim(template)
        dimnames(value) <- dimnames(template)
        value
      }
    
    tmp.upper <- cut(slot(x,colorize.upper), colorcut.upper, include.lowest=TRUE)
    colormat.upper <- matform(as.numeric(tmp.upper), slot(x,colorize.upper) )
    n.upper <- matform( paste("(",x@n,")",sep="" ), x@n)

    tmp.lower <- cut(slot(x,colorize.lower), colorcut.lower, include.lowest=TRUE)
    colormat.lower <- t(matform(as.numeric(tmp.lower), slot(x,colorize.lower) ))
    n.lower <- t(matform( paste("(",x@n,")",sep="" ), x@n))

    nmarker <- ncol(colormat.upper)

    colormat.final <- matrix(NA, nrow=nmarker, ncol=nmarker)
    dimnames(colormat.final) <- dimnames(colormat.upper)

    lower.tri <- (row(colormat.upper) - col(colormat.upper)) > 0
    upper.tri <- (col(colormat.upper) - row(colormat.upper)) > 0

    colormat.final[upper.tri] <- as.numeric(colormat.upper[upper.tri])
    colormat.final[lower.tri] <- as.numeric(colormat.lower[lower.tri])+ length(colors.upper)

    #
    # color coded frame boxes
    #

    colors.final <- c(colors.upper, colors.lower)
    image(x=1:nmarker, y=1:nmarker,
          z=t(colormat.final[nmarker:1,,drop=FALSE]),
          breaks=0:length(colors.final)+0.5,
          col=colors.final, xlab="", ylab="",
          xaxt="n", yaxt="n",...)

    abline(v=-0.5 + 1:(nmarker+1))
    abline(h=-0.5 + 1:(nmarker+1))
    
    
    #
    # text in boxes
    #

    if(missing(cex))
      {
        maxheight <- max(length(lower),
                         length(upper)) *
                     strheight('W')*1.05
        maxlength <- max(sapply(lower,nchar),
                         sapply(upper,nchar)) *
                     strwidth('W')*1.05
                
        cex <- min( 1 / maxheight, 1 / maxlength ) * 1.25
      }

    par(cex=cex)

    cin <- par("cin")

    axis(3, 1:nmarker, colnames(colormat.final))
    axis(2, 1:nmarker, rev(rownames(colormat.final)))

    lineheight <- strheight("W")*1.05
    
    showtext <- function(which, upper=TRUE)
      {
        center <- lineheight * length(which)/2

        for(i in 1:length(which))
          {
            displaymat <- slot(x,which[i])
            
            ## Put the items in lower.triangle, specified by user
            if(!upper) displaymat <- t(displaymat)
            
            if( which[i]=="P-value" )
              displaymat <- format.pval(displaymat, digits=digits)
            else if (which[i]!="n")
              displaymat <- format(round(displaymat, digits=digits),digits=digits)
            
            displaymat[] <- gsub("NA.*", "", as.character(displaymat))
            
            text(x=col(colormat.final),
                 y=nmarker - row(colormat.final)+ 1 + center - lineheight*(i-1),
                 displaymat,
                 col=textcol,
                 adj=c(0.5, 1)
                 )
          }
      }
    showtext(which=upper, upper=TRUE)
    showtext(which=lower, upper=FALSE)
    

    diag.names <- diag(1, nrow=nmarker, ncol=nmarker)
    
    text(x=1:nmarker, y=nmarker:1,
         rownames(colormat.final), adj=c(0.5,0.5),
         col=textcol)

    par(cex=oldpar$cex)
    
    #
    # title
    #
    title(main="Linkage Disequilibrium\n")


    ## Color scale calculations
    levs.lower <- levels(tmp.lower)
    levs.lower <- gsub( ",", ",\n ", levs.lower)
    
    levs.upper <- levels(tmp.upper)
    levs.upper <- gsub( ",", ",\n ", levs.upper)

    ##
    ## Lower color scale
    ##
    par(cex=1.0)
    par(mar=c(b=1.1, l=4.1, t=1.1, r=0.5))

    vals.lower <- 1:nlevels(tmp.lower)
    image(z=matrix(vals.lower,ncol=1), col=colors.lower, xaxt="n", yaxt="n")

    width <- par("usr")[2] - par("usr")[1] 
    cex.scale <- min(
                     c( 0.95 / length(levs.lower) / strwidth(levs.lower) ,
                        0.95 / length(levs.upper) / strwidth(levs.upper) )
                     )

    top    <- par("usr")[4]
    bottom <- par("usr")[3]
    center <- (top - bottom)/2 + bottom 
    
    text(x=(vals.lower-1)/(length(vals.lower)-1),
         y=rep(center,length(vals.lower)),
         labels=levs.lower, adj=c(0.5,0.5), cex=cex.scale,
         col=textcol)
    mtext(lower, side=3, las=1.1, cex=0.75)

    ##
    ## Upper color scale 
    ## 
    par(cex=1.0)
    par(mar=c(b=1.1, l=1.1, t=1.1, r=1.1))

    vals.upper <- 1:nlevels(tmp.upper)
    image(z=matrix(vals.upper,ncol=1), col=colors.upper, xaxt="n", yaxt="n")

    text(x=(vals.upper-1)/(length(vals.upper)-1),
         y=rep(center,length(vals.upper)),
         labels=levs.upper, adj=c(0.5,0.5), cex=cex.scale,
         col=textcol)
    mtext(upper, side=3, las=1, cex=0.75)
    
    invisible(colormat.final)
  }

if(!isGeneric("plot"))
  setGeneric("plot", function(x, y,...)
           standardGeneric("plot"))

setMethod("plot", "LD", plot.LD)
